@@ -35,8 +35,6 @@ constexpr int ASS_BORDER_STYLE_OUTLINE = 1; // Outline + drop shadow
 constexpr int ASS_BORDER_STYLE_BOX = 3; // Box + drop shadow
 constexpr int ASS_BORDER_STYLE_SQUARE_BOX = 4; // Square box + outline
 
-constexpr int ASS_FONT_ENCODING_AUTO = -1;
-
 // Convert RGB/ARGB to RGBA by also applying the opacity value
 COLOR::Color ConvColor(COLOR::Color argbColor, int opacity = 100)
 {
@@ -110,7 +108,7 @@ void CDVDSubtitlesLibass::Configure()
                                     XFILE::DIR_FLAG_NO_FILE_DIRS | XFILE::DIR_FLAG_NO_FILE_INFO);
   }
   // Get temporary fonts
-  if (XFILE::CDirectory::Exists(UTILS::FONT::FONTPATH::SYSTEM, false))
+  if (XFILE::CDirectory::Exists(UTILS::FONT::FONTPATH::TEMP, false))
   {
     XFILE::CDirectory::GetDirectory(
         UTILS::FONT::FONTPATH::TEMP, items, UTILS::FONT::SUPPORTED_EXTENSIONS_MASK,
@@ -398,10 +396,6 @@ void CDVDSubtitlesLibass::ApplyStyle(const std::shared_ptr<struct style>& subSty
     // text line to become more discontinuous (test on LibAss 15.1)
     style->Spacing = 0;
 
-    // Set automatic paragraph direction (not VSFilter-compatible)
-    // to fix wrong RTL text direction when there are unicode chars
-    style->Encoding = ASS_FONT_ENCODING_AUTO;
-
     bool isFontBold =
         (subStyle->fontStyle == FontStyle::BOLD || subStyle->fontStyle == FontStyle::BOLD_ITALIC);
     bool isFontItalic =
@@ -456,7 +450,7 @@ void CDVDSubtitlesLibass::ApplyStyle(const std::shared_ptr<struct style>& subSty
           ConvColor(subStyle->shadowColor, subStyle->shadowOpacity); // Set the box shadow color
       style->Shadow = (10.00 / 100 * subStyle->shadowSize) * scale; // Set the box shadow size
       // By default a box overlaps the other, then we increase a bit the line spacing
-      lineSpacing = 6.0;
+      lineSpacing = 8.0 * scaleDefault;
     }
     else if (subStyle->borderStyle == BorderType::SQUARE_BOX)
     {
@@ -468,7 +462,9 @@ void CDVDSubtitlesLibass::ApplyStyle(const std::shared_ptr<struct style>& subSty
       style->Shadow = 4 * scale; // Space between the text and the box edges
     }
 
-    ass_set_line_spacing(m_renderer, lineSpacing);
+    // ass_set_line_spacing do not scale, so we have to scale to frame size
+    ass_set_line_spacing(m_renderer,
+                         lineSpacing / playResY * static_cast<double>(opts.frameHeight));
 
     style->Blur = (10.00 / 100 * subStyle->blur);
 
