@@ -177,8 +177,12 @@ void TranslateEscapeChars(std::string& text)
 {
   if (text.find('&') != std::string::npos)
   {
-    StringUtils::Replace(text, "&lrm;", u8"\u200e");
-    StringUtils::Replace(text, "&rlm;", u8"\u200f");
+    // The specs says to use unicode
+    // U+200E for "&lrm;" and U+200F for "&rlm;"
+    // but libass rendering assume the text as left-to-right,
+    // to display text in the right order we have to use embedded codes
+    StringUtils::Replace(text, "&lrm;", u8"\u202a");
+    StringUtils::Replace(text, "&rlm;", u8"\u202b");
     StringUtils::Replace(text, "&#x2068;", u8"\u2068");
     StringUtils::Replace(text, "&#x2069;", u8"\u2069");
     StringUtils::Replace(text, "&amp;", "&");
@@ -1113,21 +1117,33 @@ void CWebVTTHandler::ConvertAddSubtitle(std::vector<subtitleData>* subList)
   // Convert tags and apply the CSS Styles converted
   ConvertSubtitle(m_subtitleData.text);
 
-  if (!subList->empty())
+  if (m_lastSubtitleData)
   {
-    // Youtube WebVTT can have multiple cues with same time, text and position
-    // sometimes with different css color but only last cue will be visible
-    // this cause unexpected results on screen so we keep only the last one
-    const subtitleData& prevSub = subList->back();
-    if (prevSub.startTime == m_subtitleData.startTime &&
-        prevSub.stopTime == m_subtitleData.stopTime && prevSub.textRaw == m_subtitleData.textRaw &&
-        prevSub.cueSettings == m_subtitleData.cueSettings)
+    // Check for same subtitle data
+    if (m_lastSubtitleData->startTime == m_subtitleData.startTime &&
+        m_lastSubtitleData->stopTime == m_subtitleData.stopTime &&
+        m_lastSubtitleData->textRaw == m_subtitleData.textRaw &&
+        m_lastSubtitleData->cueSettings == m_subtitleData.cueSettings)
     {
-      subList->pop_back();
+      if (subList->empty())
+      {
+        // On segmented WebVTT, it can happen that the last subtitle entry is sent
+        // on consecutive demux packet. Hence we avoid showing overlapping subs.
+        return;
+      }
+      else
+      {
+        // Youtube WebVTT can have multiple cues with same time, text and position
+        // sometimes with different css color but only last cue will be visible
+        // this cause unexpected results on screen, so we keep only the current one
+        // and delete the previous one.
+        subList->pop_back();
+      }
     }
   }
 
   subList->emplace_back(m_subtitleData);
+  m_lastSubtitleData = std::make_unique<subtitleData>(m_subtitleData);
 }
 
 void CWebVTTHandler::LoadColors()
